// -*- mode: C++; c-file-style: "Stroustrup" -*-
//
// This file is part of krot,
// a program for the simulation, assignment and fit of HRLIF spectra.
//
// Copyright (C) 1998 Jochen Kpper
//
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along with
// this program; see the file License. if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 , USA
//
// If you use this program for your scientific work, please cite it according to
// the file CITATION that comes with the package.
//
// $Id: dataset.cc,v 1.6 1998/10/09 17:57:27 jochen Exp $


#include "dataset.h"
#include "krotData.h"

#include <qstring.h>

#include <kapp.h>

#include <sys/types.h>
#include <cstdio>
#include <vector>



double DataSet::operator[]( int64_t freq ) const
{
    if( freq >= strt && freq <= stp ) {
	double val = data[ freq-strt ] - mn;
	return ( val * scalg ) / mx;
    } else
	return 0;
}



DataSet& DataSet::operator=( const DataSet& set )
{
    // check for object identity
    if(this == &set)
	return *this;

  // free old memory
    delete [] data;
    // and allocate new mem
    data  = new int64_t[ set.siz ];

    // copy all data
    stp  = set.stp;
    mx   = set.mx;
    mn   = set.mn;
    nam  = set.nam;
    siz  = set.siz;
    strt = set.strt;
    for( unsigned int i=0; i<siz; i++ )
	data[i] = set.data[i];

    return *this;
}



bool DataSet::load( const QString& name, const int type )
{
    KROT_LAUNCH( "Launching DataSet::load" );
    bool ret;
    switch(type) {
    case LINES_DAT_DATA:
	ret = loadLinesDat( name );
	break;
    case XY_DATA:
	ret = loadXY( name );
	break;
    case UNKNOWN_DATA:
    default:
	ret = false;
	break;
    }
    return ret;
}



bool DataSet::save( const QString& name, const int type ) const
{
    KROT_LAUNCH( "Launching DataSet::save" );
    switch( type ) {
    case XY_DATA:
	return saveXY( name );
	break;
    case UNKNOWN_DATA:
    default:
	return false;
	break;
    }
    return false;
}



bool DataSet::saveXY( const QString& name ) const
{
    KROT_LAUNCH( "Launching DataSet::saveXY" );
    QFile file( name );
    if( ! file.open( IO_ReadOnly ) )
	return false;
    else {
	QTextStream stream( &file );
	stream.width( 20 );
	stream.flags( QTextStream::fixed );
	for(unsigned int i=0; i<siz; i++) {
	    stream << strt + i*dlta << data[i] << endl;
	}
    }
    return true;
}



void DataSet::setName( const QString& name )
{
    KROT_LAUNCH( "Launching DataSet::setName" );
    nam = name;
    // check wether this name already exists
    mapSimulation::const_iterator sim  = krotData->simulations().begin();
    mapSpectrum::const_iterator   spec = krotData->spectra().begin();
    int i = 0;
    do {
	char buf[8];
	while( krotData->spectra().end() != spec )
	    if( nam != spec->first )
		spec++;
	    else {
		i++;
		sprintf(buf,"<%d>",i);
		nam = nam + buf;
		sim  = krotData->simulations().begin();
		spec = krotData->spectra().begin();
	    }
	while( krotData->simulations().end() != sim )
	    if( nam != sim->first )
		sim++;
	    else {
		i++;
		sprintf(buf,"<%d>",i);
		nam = name + buf;
		sim  = krotData->simulations().begin();
		spec = krotData->spectra().begin();
	    }
    } while( ( krotData->spectra().end() != spec )
	     || ( krotData->simulations().end() != sim ) );
    KROT_DEBUG1( "Name: %s", name.data() );
    return;
}



//--------------------------------------------------------------------------------------------------
// These are declared, but shouldn't be defined here !
// These should be abstract functions, but we cannot make DataSet an abstract
// class. Also compiler complain on missing references when they are not
// defined, although they are not used, since they are overloaded in the
// approbiate derived classes (Jochen).

bool DataSet::loadLinesDat( const QString& )
{
    KROT_ERROR( KDEBUG_KROT_GENERAL, "DataSet::loadLinesDat should be abstract!" );
    return false;
}



bool DataSet::loadXY( const QString& )
{
    KROT_ERROR( KDEBUG_KROT_GENERAL, "DataSet::loadXY should be abstract!" );
    return false;
}
