// -*- mode: C++; c-file-style: "Stroustrup" -*-
//
// This file is part of krot,
// a program for the simulation, assignment and fit of HRLIF spectra.
//
// Copyright (C) 1998 Jochen Kpper
//
// This program is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free Software
// Foundation; either version 2 of the License, or (at your option) any later
// version.
//
// This program is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for
// more details.
//
// You should have received a copy of the GNU General Public License along with
// this program; see the file License. if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 , USA
//
// If you use this program for your scientific work, please cite it according to
// the file CITATION that comes with the package.
//
// $Id: drawarea.cc,v 1.9 1998/10/09 14:49:20 jochen Exp $



#include "drawarea.h"
#include "krot.h"
#include "krotData.h"
#include "lineSelection.h"

#include <qcursor.h>
#include <qevent.h>
#include <qpainter.h>
#include <qwidget.h>

#include <sys/types.h>
#include <iostream.h>



DrawArea::DrawArea( KRotData *krotdata, QWidget *parent, const char *name )
    : QWidget( parent, name ),
      averaging( 1 ),
      krotData( krotdata )
{
    KROT_LAUNCH( "Launching DrawArea constructor" );
    setCursor( crossCursor );
    setMouseTracking( true );
    lineSelection = new DialogLineSelection( krotData );
    connect( this, SIGNAL( lineSelectionRange( const int64_t, const int64_t ) ),
	     lineSelection, SLOT( setRange( const int64_t, const int64_t ) ) );
    return;
}



void DrawArea::mouseMoveEvent( QMouseEvent *event )
{
    KROT_LAUNCH_VERBOSE( "Launching DrawArea::mouseMoveEvent" );
    emit freq( event->x() * krotData->averaging() + start );
    return;
}



void DrawArea::mousePressEvent( QMouseEvent *event )
{
     KROT_LAUNCH_VERBOSE( "Launching DrawArea::mousePressEvent" );
     int button = event->button();
     // left button starts selection of sim-lines
     if( button & LeftButton )
	 posPressed[0] = event->pos();
     // middle button makes assignment
     if( button & MidButton ) {
	 AssignedFrequency ass( lineSelection->selection() );
	 ass.frequency() = start + event->x() * krotData->averaging();
	 emit assignment( ass );
     }
     // right button does nothing yet
     return;
}



void DrawArea::mouseReleaseEvent( QMouseEvent *event )
{
    KROT_LAUNCH_VERBOSE( "Launching DrawArea::mouseReleaseEvent" );
    int button = event->button();
    // left button stops selection of sim-lines
    if( button & LeftButton ) {
	int one = start + posPressed[0].x() * krotData->averaging();
	int two = start + event->x() * krotData->averaging();
	if( one <= two )
	    emit lineSelectionRange( one, two );
	else
	    emit lineSelectionRange( two, one );
    }
    // other buttons dont have a meaning here
    return;
}



void DrawArea::paintEvent( QPaintEvent *event )
{
    KROT_LAUNCH_VERBOSE( "Launching DrawArea::paintEvent" );
    KROT_DEBUG2_VERBOSE( "window height: %d, width: %d", height(), width() );
    QPainter paint;
    paint.begin( this );
    if( krotData->empty() ) {
	paint.fillRect( event->rect(), paint.backgroundColor() );
	paint.setPen( red );
	paint.drawText( 50, 50, i18n( "no spectrum" ) );
    } else {
	paint.fillRect( event->rect(), paint.backgroundColor() );
	mapSpectrum::const_iterator spec = krotData->spectra().begin();
	while( krotData->spectra().end() != spec ) {
	    paint.setPen( spec->second.color() );
	    paint.moveTo( event->rect().left(), computeIntensity( spec->second, start ) );
	    for( int x = event->rect().left() + 1; x <= event->rect().right(); x++ ) {
		paint.lineTo( x, computeIntensity( spec->second, start + x * krotData->averaging() ) );
	    }
	    spec++;
	}
	mapSimulation::const_iterator sims = krotData->simulations().begin();
	while( krotData->simulations().end() != sims ) {
	    paint.setPen( sims->second.color() );
	    paint.moveTo( event->rect().left(), height() - static_cast<int>( height() * sims->second[ start ] ) -1 );
	    for( int x = event->rect().left() + 1; x <=  event->rect().right(); x++ )
		paint.lineTo( x, computeIntensity( sims->second, start + x * krotData->averaging() ) );
	    sims++;
	}
    }
    paint.end();
    return;
}



void DrawArea::resizeEvent( QResizeEvent * )
{
    KROT_LAUNCH_VERBOSE( "Launching DrawArea::resizeEvent." );
    checkStartFreq();
    return;
}



void DrawArea::setStart( int64_t val )
{
    KROT_LAUNCH_VERBOSE( "Launching DrawArea::setStartFreq" );
    int64_t beg = krotData->start();
    int64_t end = krotData->stop() - width();
    val -= width()/2;
    val *= krotData->averaging();
    start = val < beg ? beg : val > end  ? end : val;
    // the plain repaint is faster than scrolling the widget. ( Sep. 98, Qt-1.40, Jochen )
    repaint( true );
#if SIZEOF_LONG >= 8
    KROT_DEBUG1_VERBOSE( "New DrawArea start frequency = %ld", start );
#else
    KROT_DEBUG1_VERBOSE( "New DrawArea start frequency = %lld", start );
#endif
    return;
}
